/*
 * Copyright (c) 2008, Harald Kuhr
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name "TwelveMonkeys" nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.twelvemonkeys.image;

import java.awt.image.*;

/**
 * Adjusts the brightness of an image.
 *
 * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
 * @author last modified by $Author: haku $
 *
 * @version $Id: //depot/branches/personal/haraldk/twelvemonkeys/release-2/twelvemonkeys-core/src/main/java/com/twelvemonkeys/image/BrightnessFilter.java#1 $
 *
 * @deprecated Use BrightnessContrastFilter instead.
 * @see BrightnessContrastFilter
 *
 */

class BrightnessFilter extends RGBImageFilter {

    // This filter can filter IndexColorModel, as it is does not depend on
    // the pixels' location
    {
        canFilterIndexColorModel = true;
    }

    private float mAmmount = 0.7f; // Same as Color.brighter/darker

    /**
     * Creates a BrightnessFilter
     */

    public BrightnessFilter() {
    }

    /**
     * Creates a BrightnessFilter
     */

    public BrightnessFilter(float pAmmount) {
        if (pAmmount < 0f) {
            pAmmount = 1f / (1f + pAmmount);
        }
        else {
            pAmmount = 1f - pAmmount;
        }
        mAmmount = pAmmount;
    }

    /**
     * Subclasses must specify a method to convert a single input pixel
     * in the default RGB ColorModel to a single output pixel.
     * @param pX
     * @param pY the coordinates of the pixel
     * @param pARGB the integer pixel representation in the default RGB
     *            color model
     * @return a filtered pixel in the default RGB color model.
     * @see ColorModel#getRGBdefault
     * @see #filterRGBPixels
     */
    public int filterRGB(int pX, int pY, int pARGB) {
        // See Color.darker() and Color.brighter()
        // This code is borrowed

        // Get color components
        int r = pARGB >> 16 & 0xFF;
        int g = pARGB >> 8 & 0xFF;
        int b = pARGB & 0xFF;

        // 1. black.brighter() should return grey
        // 2. applying brighter to blue will always return blue, brighter
        // 3. non pure color (non zero rgb) will eventually return white

        int i = (int) (1.0 / mAmmount);

        if (r == 0 && g == 0 && b == 0) {
            return (pARGB & 0xFF000000) | (i << 16) | (i << 8) | i;
        }

        if (r < i && i > 0) {
            r = i;
        }
        if (g < i && i > 0) {
            g = i;
        }
        if (b < i && i > 0) {
            b = i;
        }

        /*
        if (r > 0 && r < i) {
            r = i;
        }
        if (g > 0 && g < i) {
            g = i;
        }
        if (b > 0 && b < i) {
            b = i;
        }
        */
        

        //System.out.println("BEFORE: " + Integer.toHexString((pARGB & 0xFF000000) | (r << 16) | (g << 8) | b));

        r = Math.min((int) (r / mAmmount), 255);
        g = Math.min((int) (g / mAmmount), 255);
        b = Math.min((int) (b / mAmmount), 255);

        //System.out.println("AFTER: " + Integer.toHexString((pARGB & 0xFF000000) | (r << 16) | (g << 8) | b));

        // Added by me:
        /*
        r = Math.max((int) (r / mAmmount), 0);
        g = Math.max((int) (g / mAmmount), 0);
        b = Math.max((int) (b / mAmmount), 0);
        */

        // Return ARGB pixel, leave transparency as is
        return (pARGB & 0xFF000000) | (r << 16) | (g << 8) | b;
    }

}
