/*
 * Copyright (c) 2008, Harald Kuhr
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name "TwelveMonkeys" nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.twelvemonkeys.image;

import java.awt.image.RGBImageFilter;


/**
 * Adjusts the contrast of an image.
 *
 * @author <a href="mailto:harald.kuhr@gmail.com">Harald Kuhr</a>
 * @author last modified by $Author: haku $
 *
 * @version $Id: //depot/branches/personal/haraldk/twelvemonkeys/release-2/twelvemonkeys-core/src/main/java/com/twelvemonkeys/image/ContrastFilter.java#1 $
 *
 * @deprecated Use BrightnessContrastFilter instead.
 * @see BrightnessContrastFilter
 */

class ContrastFilter extends RGBImageFilter {

    // This filter can filter IndexColorModel, as it is does not depend on
    // the pixels' location
    {
        canFilterIndexColorModel = true;
    }

    private int[] mLUT = null;

    //private float mAmmount = 0.3f;

    /**
     * Creates a ContrastFilter
     */

    public ContrastFilter() {
        this(0.3f);
    }

    /**
     * Creates a ContrastFilter
     */

    public ContrastFilter(float pAmmount) {
        mLUT = createLut(pAmmount);
    }

    private static int[] createLut(float pAmmount) {
        int[] lut = new int[256];

        // Hmmm.. This approximates Photoshop values.. Not good though..
        double amt = pAmmount > 0 ? Math.pow(pAmmount, 7d) * 127d : pAmmount;

        for (int i = 0; i < 256; i++) {
            lut[i] = clamp(127 + (int) ((i - 127) * (amt + 1d)));
        }

        // Special case, to ensure only primary colors for max contrast
        if (pAmmount == 1f) {
            lut[127] = 0;
        }

        return lut;
    }

    private static int clamp(int i) {
        if (i < 0) {
            return 0;
        }
        if (i > 255) {
            return 255;
        }
        return i;
    }

    /**
     * Filters one pixel
     *
     * @param pX x
     * @param pY y
     * @param pARGB pixel value in default color space
     *
     * @return the filtered pixel value in the default color space
     */

    public int filterRGB(int pX, int pY, int pARGB) {
        // Get color components
        int r = pARGB >> 16 & 0xFF;
        int g = pARGB >> 8 & 0xFF;
        int b = pARGB & 0xFF;

        // Scale to new contrast
        r = mLUT[r];
        g = mLUT[g];
        b = mLUT[b];
        //r = scale(r, mAmmount);
        //g = scale(g, mAmmount);
        //b = scale(b, mAmmount);

        // Return ARGB pixel, leave transparency as is
        return (pARGB & 0xFF000000) | (r << 16) | (g << 8) | b;
    }

    /*
    private final static int scale(int pComponent, float pAmmount) {
        // TODO: This function is not correct!
        //f(x,-1) = 127
        //f(x,0) = x
        //f(x,1) = x <= 127 -> 0, x > 127 -> 255 (unsolvable?)
        //f(x,y) = ??

        pComponent = 127 + (int) ((pComponent - 127) * (pAmmount + 1f));

        if (pComponent > 255) {
            pComponent =  255;
        }
        else if (pComponent < 0) {
            pComponent = 0;
        }

        return pComponent;
    }
    */

}
